/*******************************************************************************
 *	BdsDataClient1.cpp	BDS API example code for a Data Client
 *				T.Barnaby,	BEAM Ltd,	2008-09-02
 *******************************************************************************
 *
 *	This is a very basic example of using the BdsApi from a data access
 *	perspective. It is designed to give an overview of using the API.
 *	This program gets data in the BKNAS format.
 */
#include <iostream>
#include <stdio.h>
#include <BdsD.h>
#include <BdsC.h>

using namespace Bds;
using namespace std;


// Function to read some data
BError bdsTest(DataAccess& bds){
	BError			err;
	Selection		selection;
	DataInfo		dataInfo;
	DataHandle		dataHandle;
	BArray<BUInt8>		data;
	
	// Set up selection
#ifndef ZAP
	selection.startTime.setString("2002-01-01T00:00:00.000000");
	selection.endTime.setString("2002-01-01T00:01:00.000000");
#else
	selection.startTime.setString("2002-01-01T23:59:00.000000");
	selection.endTime.setString("2002-01-02T00:01:00.000000");
#endif
	selection.channels.append(SelectionChannel("BN", "EKA", "", ""));

	// Get list of all data available for the selection		
	if(err = bds.dataSearch(selection, dataInfo)){
		return err.set(1, BString("Error: Searching for data: ") + err.getString());
	}

	// We should now choose which set of data we would like from the list, here we just
	// choose the first entry and get the data in appropriate format.
	if(!dataInfo.channels.size())
		return err.set(1, "No data found");
	
	if(err = bds.dataOpen(dataInfo, "r", "IMS", dataHandle)){
		return err;
	}
	
	while(1){
		if(err = bds.dataFormattedRead(dataHandle, 1024, data)){
			return err;
		}

		if(data.size() == 0)
			break;
		
		fwrite(data.data(), 1, data.size(), stdout);
	}
	
	return err;
}

int main(int argc, char** argv){
	BError			err;
	BString			hostName;
	DataAccess		bds;

	hostName = getenv("BDS_HOST");
	if(hostName == "")
		hostName = "localhost";
	
	if(argc == 2)
		hostName = argv[1];

	// Connect to the DataAccess service
	if(err = bds.connectService(BString("//") + hostName + "/bdsDataAccess")){
		cerr << "Error: " << err.getString() << "\n";
		return 1;
	}

	// Connect to service
	if(!err)
		err = bds.connect("test", "beam00");
	
	// Run a normal data gathering as a normal data access client would.
	if(!err)
		err = bdsTest(bds);
	
	if(err){	
		cerr << "Error: " << err.getString() << "\n";
		return 1;
	}
		
	return 0;
}
