/*******************************************************************************
 *	BdsMetaData1.cpp	BDS API example code for a Meta Data Client
 *				T.Barnaby,	BEAM Ltd,	2009-07-01
 *******************************************************************************
 *
 *	This is a very basic example of using the BdsApi from a meta data access
 *	perspective. It is designed to give an overview of using the API.
 *	This program gets information on the data channels.
 */
#include <iostream>
#include <iomanip>
#include <stdio.h>
#include <BdsD.h>
#include <BdsC.h>

using namespace Bds;
using namespace std;


// Function to read some data
BError bdsTest1(DataAccess& bds){
	BError			err;
	Selection		selection;
	BIter			i;
	BUInt			n;
	BList<Station>		stations;
	
	// Set up selection
	selection.startTime.setString("2008-01-03T00:00:00.000000");
	selection.endTime.setString("2008-01-03T00:01:00.000000");
	selection.channels.append(SelectionChannel("BN", "EKA", "", ""));

	// Get list of stations available
	if(err = bds.stationGetList(selection, stations)){
		return err.set(1, BString("Error: Getting stations: ") + err.getString());
	}

	// This displays some of the information available	
	for(stations.start(i), n = 0; !stations.isEnd(i); stations.next(i), n++){
		Station&	c = stations[i];
		
		cout << n << ": Station: " << c.name << " Type: " << c.type << "\n";
		cout << "    " << "Description: " << c.description
			<< " Number of stations " << c.channels.number() << "\n";
	}
	
	
	return err;
}

int main(int argc, char** argv){
	BError			err;
	BString			hostName;
	DataAccess		bds;

	hostName = getenv("BDS_HOST");
	if(hostName == "")
		hostName = "localhost";
	
	if(argc == 2)
		hostName = argv[1];

	// Connect to the DataAccess service
	if(err = bds.connectService(BString("//") + hostName + "/bdsDataAccess")){
		cerr << "Error: " << err.getString() << "\n";
		return 1;
	}
	
	// Connect to service
	if(!err)
		err = bds.connect("test", "beam00");
	
	// Run a normal data gathering as a normal data access client would.
	if(!err)
		err = bdsTest1(bds);
	
	if(err){
		cerr << "Error: " << err.getString() << "\n";
		return 1;
	}
		
	return 0;
}
