/*******************************************************************************
 *	BdsDataFile.h	Data File Access
 *			T.Barnaby,	BEAM Ltd,	2008-06-24
 *******************************************************************************
 */
#ifndef BdsDataFile_H
#define	BdsDataFile_H

#include <BString.h>
#include <BFile.h>
#include <BTimeStamp.h>
#include <BdsLib.h>
#include <BdsD.h>

namespace Bds {

class DataBlockPos {
public:
				DataBlockPos(BTimeStamp startTime = 0, BTimeStamp endTime = 0, BUInt64 position = 0, BUInt order = 0) : startTime(startTime), endTime(endTime), position(position), order(order){}
	int			operator<(const DataBlockPos& b) const { return startTime < b.startTime;	}
	BTimeStamp		startTime;
	BTimeStamp		endTime;
	BUInt64			position;
	BUInt			order;
};

/// This class defines the interface for generic data file access
class DataFile {
public:
	enum DataOrder		{ DataOrderUnknown, DataOrderAll, DataOrderSample, DataOrderChannel };
	enum Features		{ FeatureNone = 0x00, FeatureCanWrite = 0x01, FeatureCanRead = 0x02 };
	enum OptionsList	{ OptionNone = 0x00, OptionValidate = 0x01, OptionFileNameProcess = 0x02, OptionValidateCorruptions = 0x04, OptionReorder = 0x08, OptionDeleteDuplicates = 0x10, OptionInfoExtra = 0x20 };
	typedef int Options;

				DataFile();
	virtual			~DataFile();
	
	virtual void		init();								///< Initialise
	virtual BError		open(BString fileName, BString mode);				///< Open the file for read or write
	virtual BError		close();							///< Close the file

	virtual BError		setFormat(BString format);					///< Set the sub-format
	virtual	BString		getFileName();							///< Return the file name
	virtual DataOrder	getDataOrder();							///< Get the expected order of writing data, by sample or by channel
	virtual int		getFeatures();							///< Get bitmask of supported features
	
				// Write routines
	virtual BError		setInfo(DataInfo& dataInfo, ChannelInfos& channelInfos);		///< Set information on data for write
	virtual	BError		start(BUInt channel, BUInt segment);					///< Start next segment of data
	virtual BError		writeData(DataBlock& data);						///< Write a block of data
	virtual BError		end();									///< End segment

				// Read routines
	virtual BError		fileNameProcess();							///< Parse the file name for a date/time
	virtual BError		getFormat(BString& format);						///< Get sub-format
	virtual BError		getInfo(DataInfo& dataInfo, Options options, BList<BError>& errors);	///< Get info on data
	virtual BError		seekBlock(BUInt32 channel, BUInt segment, BTimeStamp time, BUInt32& blockNumber, BUInt64& sampleNumber, DataBlock& data);	///< Find requested block on given channel given a time
	virtual BError		readData(BUInt32 channel, BUInt segment, BUInt32 blockNumber, DataBlock& dataBlock);	///< Read a block
	
	static DataFormat	getFormats();							///< Get list of supported formats

protected:
	BString			ofileName;
	BString			omode;
	BTimeStamp		ofileNameTime;
	BFile			ofile;
	BString			oformat;
};

}
#endif
