/*******************************************************************************
 *	BString.hxx		BString Handling
 *				T.Barnaby,	BEAM Ltd, 29/10/91
 *******************************************************************************
 */
#ifndef BSTRING_H
#define BSTRING_H	1

#include	<BTypes.h>
#include	<BRefData.h>
#include	<BList.h>
#include	<BArray.h>
#include	<iostream>

class BString {
public:
			BString();
			BString(const BString& string);
			BString(const char* str);
			BString(const char* str, unsigned int len);
			BString(char ch);
			BString(int v);
			BString(unsigned int v);
			BString(long v);
			BString(unsigned long long);
			BString(double v);

			~BString();

	static		BString convert(char ch);			///< Converts char to string
	static		BString	convert(int value);			///< Converts int to string
	static		BString	convert(unsigned int value);		///< Converts uint to string
	static		BString	convert(long value);			///< Converts long to string
	static		BString	convert(double value, int eFormat = 0);			///< Converts double to string
	static		BString	convert(unsigned long long value);	///< Converts u long long to string
	static		BString	convertHex(int value);			///< Converts int to string as hex value
	static		BString	convertHex(unsigned int value);		///< Converts uint to string as hex value

	BString		copy();						///< Return an independant copy
	
	int		len() const;						///< Length of string
	const char*	retStr() const;						///< Ptr to char* representation
	char*		retStrDup() const;					///< Ptr to newly malloc'd char*
	int		retInt() const;						///< Return string as a int
	unsigned int	retUInt() const;					///< Return string as a int
	double		retDouble() const;					///< Return string as a double

	int		compare(const BString& string) const;			///< Compare strings
	int		compareWild(const BString& string) const;		///< Compare string to string with wildcards 
	int		compareWildExpression(const BString& string) const;	///< Compare string to space deliminated patterns
	BString		append(const BString& str) const;			///< Append a string
	BString&	truncate(int len);					///< Truncate to length len
	BString&	pad(int len);						///< Pad to length len
	
	BString&	toUpper();						///< Convert to uppercase
	BString&	toLower();						///< Convert to lowercase
	void		removeNL();						///< Remove if present NL from last char 
	BString		justify(int leftMargin, int width);			///< Justify the string to the given width
	
	BString		subString(int start, int len) const;			///< Returns substring
	int		del(int start, int len);				///< Delete substring
	int		insert(int start, BString str);				///< Insert substring
	void		printf(const char* fmt, ...) ;				///< Formated print into the string
	int		find(char ch) const;					///< Find ch in string searching forwards
	int		find(BString str) const;				///< Find string in string searching forwards
	int		findReverse(char ch) const;				///< Find ch in string searching backwards

	BList<BString>	getTokenList(BString separators);			///< Break string into tokens 
	BList<BString>	getTokenList(char separator);				///< Break string into tokens 
	BString		removeSeparators(BString separators);			///< Remove any char from sepatators from string
	BString		pullToken(BString terminators);				///< Pull token from start of string
	BString		pullSeparators(BString separators);			///< Pull separators from start of string
	BString		pullWord();						///< Pull a word out of the head of the string
	BString		pullLine();						///< Pull a line out of the head of the string

	// Filename operations
	BString		dirname();
	BString		basename();
	BString		extension();
	
	// Misc functions
	BUInt32		hash() const;
	
	/* Operator functions */
	BString&	operator=(const BString& string);
	char&		operator[](int pos);
	
	int		operator==(const BString& s) const{
				return (compare(s) == 0);
			}
	int		operator==(const char* s) const{
				return (compare(s) == 0);
			}
	int		operator>(const BString& s) const{
				return (compare(s) > 0);
			}
	int		operator>(const char* s) const{
				return (compare(s) > 0);
			}
	int		operator<(const BString& s) const{
				return (compare(s) < 0);
			}
	int		operator<(const char* s) const{
				return (compare(s) < 0);
			}
	int		operator>=(const BString& s) const{
				return (compare(s) >= 0);
			}
	int		operator<=(const BString& s) const{
				return (compare(s) <= 0);
			}
	int		operator!=(const BString& s) const{
				return (compare(s) != 0);
			}
	int		operator!=(const char* s) const{
				return (compare(s) != 0);
			}
	BString 	operator+(const BString& s) const {
				return append(s);
			}
	BString 	operator+(const char* s) const {
				return append(s);
			}
	BString 	operator+=(const BString& s){
				*this = append(s);
				return *this;
			}
	BString 	operator+=(const char* s){
				*this = append(s);
				return *this;
			}
			
	/* Special operators */
	BString 	operator+(char ch) const {
				return append(convert(ch));
			}
	BString 	operator+(int i) const {
				return append(convert(i));
			}
	BString 	operator+(unsigned int i) const {
				return append(convert(i));
			}
	BString 	operator+(unsigned long long i) const {
				return append(convert(i));
			}
			operator const char* () const {
				return retStr();
			}
#ifdef QSTRING_H
	// QT support
			BString(const QString& str){
#if QT_VERSION >= 0x040000
				init(str.toAscii());
#else
				init(str);
#endif
			}
#ifndef ZAP
			operator QString (){		// QString support
				return QString(retStr());
			}
#endif
#endif

	// For compatibility
	BString		field(int field) const;
	char**		fields();

protected:
	BRefData*	ostr;	

private:
	void		init(const char* str);
	int		inString(int pos) const;
	int		isSpace(char ch) const;
};

/* OStream Functions */
std::ostream& operator<<(std::ostream& o, BString& s);
std::istream& operator>>(std::istream& i, BString& s);

#ifdef BLIST_H
typedef BList<BString>	BStringList;
typedef BArray<BString>	BStringArray;

int bstringListinList(BStringList& l, BString s);

BString blistToString(const BStringList& list);		///< Convert a string list to a comma separated string
BStringList bstringToList(BString str);			///< Convert a comma separated string to a string list
BStringList charToList(const char** str);

BString barrayToString(const BStringArray& list);	///< Convert a string array to a comma separated string
BStringArray bstringToArray(BString str);		///< Convert a comma separated string to a string array
BStringArray charToArray(const char** str);

#endif

// String conversion functions
void toBString(BString& v, BString& s);
void toBString(BStringList& v, BString& s);
void toBString(BInt32& v, BString& s);
void toBString(BUInt32& v, BString& s);
void toBString(BUInt64& v, BString& s);
void toBString(BFloat64& v, BString& s);
void fromBString(BString& s, BString& v);
void fromBString(BString& s, BStringList& v);
void fromBString(BString& s, BInt32& v);
void fromBString(BString& s, BUInt32& v);
void fromBString(BString& s, BUInt64& v);
void fromBString(BString& s, BFloat64& v);

#endif
